@@ -71,9 +71,13 @@ sub read {
 
     my $c = [] ;
     $logger->trace("header: @$header") ;
-    my @stuff ;
-    my $store_stuff = sub { push @stuff, shift ;} ;
-    
+    my %stuff ;
+    my $store_stuff = sub {
+        my ($l,$nb) = @_;
+        die "undef line nb" unless defined $nb;
+        $stuff{$nb} = $l ;
+    } ;
+
     if (@$header) {
         $c = eval { $self->parse_dpkg_lines( $header, $check, 0, $store_stuff ); };
         my $e;
@@ -93,25 +97,39 @@ sub read {
           if @$c > 4; # $c contains [ line_nb, section_ref ]
     }
 
+    my $description_holder;
+    my $description_text ;
     while (@$c) {
         my ( $section_line, $section ) = splice @$c, 0, 2;
         foreach ( my $i = 0 ; $i < $#$section ; $i += 2 ) {
             my $key = $section->[$i];
             my ( $v, $l, $a, @comments ) = @{ $section->[ $i + 1 ] };
             if ( my $found = $node->find_element( $key, case => 'any' ) ) {
-                my @elt = ($found);
-                $v .= join( "\n", @stuff ) if $found eq 'Subject';
-                my @v = ( $found eq 'Description' ) ? ( split /\n/, $v, 2 ) : ($v);
-                unshift @elt, 'Synopsis' if $found eq 'Description';
-                foreach (@elt) {
-                    my $sub_v = shift @v;
-                    next unless defined $sub_v;
-                    $logger->debug("storing $_  value: $sub_v");
-                    $node->fetch_element($_)->store( value => $sub_v, check => $check );
+                my $elt = $found ;
+                my $to_store = $v;
+                if ($found =~ /^Subject|Description$/) {
+                    $description_holder = $found;
+                    $elt = 'Synopsis';
+                    ($to_store, $description_text)= split /\n+/, $v, 2 ;
                 }
+
+                $logger->debug("storing $elt  value: $to_store");
+                $node->fetch_element($elt)->store( value => $to_store, check => $check );
+            }
+            else {
+                $stuff{$section_line} = "$key: $v";
             }
         }
     }
+
+    my $k = 0;
+
+    if ($description_holder) { 
+        my @desc_lines = map { my $pre = ( $k + 1 == $_ ? '' : "\n"); $k = $_; $pre.$stuff{$_} }
+            sort { $a <=> $b ;} keys %stuff ;
+        my $desc = ($description_text || '') .join( "\n", @desc_lines );
+        $node->fetch_element($description_holder)->store( value => $desc, check => $check );
+    }
     $node->fetch_element('diff')->store(join('',@$diff));
 
     return 1;
@@ -142,6 +160,8 @@ sub write {
         message => "cannot write patch $patch_file" );
     $io->binmode(":utf8");
 
+    my $subject_body;
+    my $diff;
     foreach my $elt ( $node -> get_element_name ) {
         my $elt_obj = $node->fetch_element($elt) ;
         my $type = $node->element_type($elt) ;
@@ -155,14 +175,32 @@ sub write {
             next unless defined $v and $v;
 
             if ($elt eq 'Synopsis') {
-                my $long_description = $node->fetch_element_value('Description') ;
-                $v .= "\n" . $long_description if $long_description ;
-                $io->print("Description: ");
-                $self->write_dpkg_text($io,$v) ;
+                my $synopsis = $v ;
+                my $description_body = $node->fetch_element_value('Description') ;
+                $subject_body = $node->fetch_element_value('Subject') ;
+
+                if ($description_body and $subject_body) {
+                    die "Write error: cannot have both Subject body and description body";
+                }
+                elsif ($description_body) {
+                    my $to_write = $synopsis . "\n" . $description_body ;
+                    $io->print("Description: ");
+                    $self->write_dpkg_text($io, $to_write) ;
+                }
+                elsif ($subject_body) {
+                    # DEP-3: When Subject is used, it is expected that the
+                    # long description is outside of the structured
+                    # fields.
+                    $io->print("Subject: ");
+                    $self->write_dpkg_text($io, $synopsis) ;
+                }
+                else {
+                    $io->print("Description: ");
+                    $self->write_dpkg_text($io, $synopsis) ;
+                }
             }
-            elsif ($elt eq 'Description') { } # done in Synopsis
-            elsif ($elt eq 'diff' ) {
-                $io->print($node->fetch_element_value('diff')) ;
+            elsif ($elt eq 'Description' or $elt eq 'Subject' or $elt eq 'diff') {
+                # first 2 are handled with Synopsis. diff is handled below
             }
             else {
                 $io->print("$elt: ");
@@ -171,6 +209,12 @@ sub write {
         }
     }
 
+    if ($subject_body) {
+        $io->print($subject_body."\n");
+    }
+
+    $io->print($node->fetch_element_value('diff')) ;
+
     return 1;
 }
 
