@@ -2,6 +2,8 @@ package Config::Model::Dpkg::Dependency ;
 
 use 5.10.1;
 
+use Config::Model 2.066; # for show_message
+
 use Mouse;
 use URI::Escape;
 
@@ -123,6 +125,18 @@ my %virtual_hash = map {( $_ => 1); } @virtual_list;
 use vars qw/$test_filter/ ;
 $test_filter = ''; # reserved for tests
 
+my %debian_map;
+my $version = \%Module::CoreList::version;
+
+foreach my $v (values %$version) {
+    foreach my $pm ( keys %$v ) {
+        next unless defined $pm;
+        my $k = lc($pm);
+        $k =~ s/::/-/g;
+        $debian_map{"lib$k-perl"} = $pm;
+    }
+}
+
 my $logger = get_logger("Tree::Element::Value::Dependency") ;
 
 # initialise the global config object with the default values
@@ -136,7 +150,7 @@ my $vs = $_system->versioning;
 
 my $apt_cache = AptPkg::Cache->new ;
 
-# end black magic
+# end of AptPkg black magic
 
 extends qw/Config::Model::Value/ ;
 
@@ -167,7 +181,7 @@ dependency: depend(s /\|/) eofile {
 depend: pkg_dep | variable
 
 # For the allowed stuff after ${foo}, see #702792
-variable: /\${[\w:\-]+}[\w\.\-~+]*/
+variable: /\$\{[\w:\-]+\}[\w\.\-~+]*/
 
 pkg_dep: pkg_name dep_version(?) arch_restriction(?) profile_restriction(s?) {
     my $dv = $item[2] ;
@@ -224,9 +238,13 @@ os: /(any|uclibc-linux|linux|kfreebsd|knetbsd|kopensolaris|hurd|darwin|freebsd|n
    -/x
    | /\w+/ '-' { $add_error->("bad os in architecture specification", $item[1]) ;}
 
-arch: / (any |alpha|amd64 |arm\b |arm64 |armeb |armel |armhf |avr32
-        |hppa |i386 |ia64 |lpia |m32r |m68k |mips\b |mipsel |powerpc
-        |powerpcspe |ppc64 |s390 |s390x |sh3\b |sh3eb |sh4\b |sh4eb |sparc\b |sparc64 |x32 )
+arch: / (any |alpha|amd64
+         |arm(64|eb|el|hf)?
+         |avr32 |hppa |i386 |ia64 |lpia |m32r |m68k
+         |mips(el|64el|64)?
+         |powerpc(el|spe)?
+         |ppc64\b |ppc64el |s390 |s390x
+         |sh3\b |sh3eb |sh4\b |sh4eb |sparc\b |sparc64 |x32 )
         (?=(\]| ))
       /x
       | /\w+/ { $add_error->("bad arch in architecture specification", $item[1]) ;}
@@ -321,28 +339,35 @@ sub check_debhelper_version {
     my $lintian_dep = Lintian::Relation->new( $dep_string ) ;
     $logger->debug("checking '$dep_string' with lintian");
 
-    # using mode loose because debian-control model can be used alone
-    # and compat is outside of debian-control
-    my $compat = $self->grab_value(mode => 'loose', step => "!Dpkg compat") ;
-    return unless defined $compat ;
+    # try to create compat_obj, but do not try twice (hence the exists test)
+    if (not exists $self->{_compat_obj} ) {
+        # using mode loose because debian-control model can be used alone
+        # and compat is outside of debian-control
+        my $c = $self->{_compat_obj} = $self->grab(mode => 'loose', step => "!Dpkg compat") ;
+        $c->register_dependency($self) if defined $c;
+    }
 
-    my $min_dep = Lintian::Relation->new("debhelper ( >= $compat)") ;
+    return unless defined $self->{_compat_obj};
+
+    my $compat_value = $self->{_compat_obj}->fetch;
+
+    my $min_dep = Lintian::Relation->new("debhelper ( >= $compat_value)") ;
     $logger->debug("checking if ".$lintian_dep->unparse." implies ". $min_dep->unparse);
 
     return if $lintian_dep->implies ($min_dep) ;
 
-    $logger->debug("'$dep_string' does not imply debhelper >= $compat");
+    $logger->debug("'$dep_string' does not imply debhelper >= $compat_value");
 
     # $show_rel avoids undef warnings
     my $show_rel = join(' ', map { $_ || ''} ($oper, $dep_v));
     if ($apply_fix) {
-        @$depend = ( 'debhelper', '>=', $compat ) ; # notify_change called in check_value
+        @$depend = ( 'debhelper', '>=', $compat_value ) ; # notify_change called in check_value
         $logger->info("fixed debhelper dependency from "
-            ."$dep_name $show_rel -> ".$min_dep->unparse." (for compat $compat)");
+            ."$dep_name $show_rel -> ".$min_dep->unparse." (for compat $compat_value)");
     }
     else {
         $self->{nb_of_fixes}++ ;
-        my $msg = "should be (>= $compat) not ($show_rel) because compat is $compat" ;
+        my $msg = "should be (>= $compat_value) not ($show_rel) because compat is $compat_value" ;
         $self->add_warning( $msg );
         $logger->info("will warn: $msg (fix++)");
     }
@@ -414,9 +439,8 @@ sub check_depend_chain {
             my ($dep_name, $oper, $dep_v) = @$depend ;
             $logger->debug("scanning dependency $dep_name"
                 .(defined $dep_v ? " $dep_v" : ''));
-            if ($dep_name =~ /lib([\w+\-]+)-perl/) {
-                my $pname = $1 ;
-                $ret &&= $self->check_perl_lib_dep ($apply_fix, $pname, $actual_dep, $depend,$input);
+            if ($dep_name =~ /lib[\w+\-]+-perl/) {
+                $ret &&= $self->check_perl_lib_dep ($apply_fix, $actual_dep, $depend,$input);
                 last;
             }
         }
@@ -431,15 +455,23 @@ sub check_depend_chain {
     return $ret ;
 }
 
+sub extract_cpan_version {
+    my ($self,$v) = @_;
+    return undef unless defined $v ;
+    $v =~ /^(?:\d+:)?([\d\w.-]+?)(?:(?:\+\w+)?-[\w+~]+)?$/ ;
+    return $1;
+}
+
 # called through check_depend_chain
 # does modify $input when applying fix
 sub check_perl_lib_dep {
-    my ($self, $apply_fix, $pname, $actual_dep, $depend, $input) = @_;
-    $logger->debug("called for $pname with $actual_dep with apply_fix $apply_fix");
-
+    my ($self, $apply_fix, $actual_dep, $depend, $input) = @_;
     my ( $dep_name, $oper, $dep_v ) = @$depend;
 
-    $pname =~ s/-/::/g;
+    $logger->debug("called for $dep_name with $actual_dep with apply_fix $apply_fix");
+
+    my ($old_perl_dep) = grep { $_->[0] eq 'perl' } @$input;
+    my $old_perl_versioned_dep = $old_perl_dep->[2];
 
     # The dependency should be in the form perl (>= 5.10.1) | libtest-simple-perl (>= 0.88)".
     # cf http://pkg-perl.alioth.debian.org/policy.html#debian_control_handling
@@ -448,15 +480,18 @@ sub check_perl_lib_dep {
     # because buildd will use the first available alternative
 
     # check for dual life module, module name follows debian convention...
-    my @dep_name_as_perl = Module::CoreList->find_modules(qr/^$pname$/i) ;
-    return 1 unless @dep_name_as_perl;
+    my $cpan_name = $debian_map{$dep_name};
+    return 1 unless $cpan_name ;
 
-    my $deprecated = Module::CoreList->deprecated_in($dep_name_as_perl[0]) ;
+    my $first_perl =  Module::CoreList->first_release($cpan_name) ;
+    return 1 unless $first_perl;
+
+    my $deprecated = Module::CoreList->deprecated_in($cpan_name) ;
     $logger->debug("dual life $dep_name is deprecated with perl $deprecated") if $deprecated;
-    my $removed    = Module::CoreList->removed_from($dep_name_as_perl[0]) ;
+    my $removed    = Module::CoreList->removed_from($cpan_name) ;
     $logger->debug("dual life $dep_name is removed from perl $removed") if $removed;
 
-    return 1 if (defined $dep_v && $dep_v =~ m/^\$/) ;
+    return 1 if (defined $dep_v && $dep_v =~ m/^\$/) ; # version like ${foobar}
 
     my @ideal_perl_dep = qw/perl/ ;
     my @ideal_lib_dep ;
@@ -482,16 +517,15 @@ sub check_perl_lib_dep {
 	# use oldest version only if the oldest version is NOT in oldstable
 	# unfortunately this is fragile and must be modified after each Debian
 	# release
-	if ($oldest_debian_with_lib =~ /wheezy|jessie|sid/) {
+	if ($oldest_debian_with_lib =~ /wheezy|jessie|stretch|buster|sid/) {
 		$check_v ||= $oldest_lib_version_in_debian ;
 		$logger->debug("dual life $dep_name has oldest debian $oldest_lib_version_in_debian, using $check_v");
 	}
 
-	my ($cpan_dep_v, $epoch_dep_v) ;
+	my $cpan_dep_v = $self->extract_cpan_version($check_v);
 
-	($cpan_dep_v, $epoch_dep_v) = reverse split /:/ ,$check_v if defined $check_v ;
 	my $v_decimal = Module::CoreList->first_release(
-		$dep_name_as_perl[0],
+		$cpan_name,
 		version->parse( $cpan_dep_v )
 	);
 
@@ -501,22 +535,26 @@ sub check_perl_lib_dep {
 	$v_normal =~ s/^v//;    # loose the v prefix
 	if ( $logger->is_debug ) {
 		my $dep_str = $dep_name . ( defined $check_v ? ' ' . $check_v : '' );
-		$logger->debug("dual life $dep_str aka $dep_name_as_perl[0] found in Perl core $v_normal");
+		$logger->debug("dual life $dep_str found in Perl core $v_normal (req perl is $old_perl_versioned_dep)");
 	}
 
+    if ( not defined $dep_v and $old_perl_versioned_dep ) {
+        # when alternate lib version is not defined (because the requirement is satisfied by all
+        # available versions of the lib), the actual requirement is held by the versioned dep of perl
+        # hence, it must be preserved
+		$logger->debug("preserving old perl versioned dep $old_perl_versioned_dep instead of $v_normal"
+                       ." for $dep_name");
+        $v_normal = $old_perl_versioned_dep;
+    }
+
 	my ($has_older_perl) = $self->check_versioned_dep(  ['perl', '>=', $v_normal] );
 	push @ideal_perl_dep, '>=', $v_normal if $has_older_perl;
 
-	if ( defined $dep_v ) {
-		my ($has_older_lib) = $self->check_versioned_dep(  $depend );
-		if ($has_older_perl) {
-			push @ideal_lib_dep, $dep_name;
-			push @ideal_lib_dep, '>=', $dep_v if $has_older_lib;
-		}
-	}
-	elsif ($has_older_perl or $removed or $deprecated) {
-		push @ideal_lib_dep, $dep_name;
-	}
+    if ($removed or $deprecated or $has_older_perl) {
+        my ($has_older_lib) = $self->check_versioned_dep(  $depend );
+        push @ideal_lib_dep, $dep_name;
+        push @ideal_lib_dep, '>=', $dep_v if $dep_v and $has_older_lib;
+    }
 
 	my %perl_version =  $self->get_available_version( 'perl');
 	my $has_older_perl_in_sid = ( $vs->compare( $v_normal, $perl_version{sid} ) < 0 ) ? 1 : 0;
@@ -530,8 +568,12 @@ sub check_perl_lib_dep {
         = $removed || $deprecated  ? ( \@ideal_lib_dep )
         : $has_older_perl_in_sid ? ( \@ideal_perl_dep, \@ideal_lib_dep )
         :                          ( \@ideal_lib_dep, \@ideal_perl_dep ) ;
+
 	my $ideal_dep = $self->struct_to_dep( @ordered_ideal_dep );
 
+    die "Internal error: undefined ideal dep. Please report bug with the dependencies that triggered the bug"
+        unless defined $ideal_dep;
+
 	if ( $actual_dep ne $ideal_dep ) {
 		if ($apply_fix) {
 			@$input = @ordered_ideal_dep ; # notify_change called in check_value
@@ -759,7 +801,7 @@ sub get_available_version {
     }
 
     my $url = "http://qa.debian.org/cgi-bin/madison.cgi?package=".uri_escape($pkg_name)."&text=on" ;
-    say "Connecting to qa.debian.org to check $pkg_name versions. Please wait..." ;
+    $self->instance->show_message("Connecting to qa.debian.org to check $pkg_name versions. Please wait...") ;
 	my $body = get($url);
 
 	warn "cannot get data for package $pkg_name. Check your proxy ?\n" unless defined $body ;
@@ -771,15 +813,16 @@ sub get_available_version {
 		$type =~ s/\s//g ;
 		push @res , $dist,  $available_v unless $type eq 'source';
 	}
-	say "got info for $pkg_name" ;
+    $self->instance->show_message("got info for $pkg_name") ;
 	$cache{$pkg_name} = time ." @res" ;
+    $logger->debug("pkg info is ".$cache{$pkg_name});
 	return @res;
 }
 
 # this function queries *once* madison for package info not found in cache.
 # it should be called once when parsing control file
 sub cache_info_from_madison {
-    my (@pkg_names) = @_ ;
+    my ($instance,@pkg_names) = @_ ;
 
     $logger->debug("called on @pkg_names");
 
@@ -803,7 +846,9 @@ sub cache_info_from_madison {
     }
 
     my $url = "http://qa.debian.org/cgi-bin/madison.cgi?package=".join('+',map { uri_escape($_) } @needed)."&text=on" ;
-    say "Connecting to qa.debian.org to check ", scalar @needed, " package versions. Please wait..." ;
+    $instance->show_message(
+        "Connecting to qa.debian.org to check ", scalar @needed, " package versions. Please wait..."
+    );
 	my $body = get($url);
 
 	warn "cannot get data from madison. Check your proxy ?\n" unless defined $body ;
@@ -816,12 +861,13 @@ sub cache_info_from_madison {
 		$res{$name} ||= [] ;
 		push @{$res{$name}} , $dist,  $available_v unless $type eq 'source';
 	}
-	say "got info for $necessary packages: ", join(' ',sort keys %res) ;
+	$instance->show_message( "Got info from qa.debian.org for $necessary packages.") ;
 	foreach my $pname (keys %res) {
 		$cache{$pname} = time ." ".join(' ',@{$res{$pname}}) ;
 	}
 }
 
+
 __PACKAGE__->meta->make_immutable;
 
 1;
