@@ -5,7 +5,7 @@
     ],
     'class_description' => 'Model of Debian source package files (e.g debian/control, debian/copyright...)',
     'copyright' => [
-      '2010,2011 Dominique Dumont'
+      '2010-2015 Dominique Dumont',
     ],
     'element' => [
       'my_config',
