@@ -254,7 +254,7 @@ A package including binaries from grub2 and loadlin would have this field in its
 if ($@) { CORE::warn "cannot fix without Text::Autoformat"}
 else {
         import Text::Autoformat ;
-        $_ = autoformat($_) ;
+        $_ = autoformat($_, {all => 1}) ;
 	chomp;
 }',
             'msg' => 'Line too long in description'
