@@ -78,7 +78,7 @@ area/section if the package is in the contrib or non-free archive areas.
       },
       'XS-Testsuite',
       {
-        'description' => 'Enable a testsuite to be used with this package. For more details see L<README.package-tests.rst|https://anonscm.debian.org/cgit/autopkgtest/autopkgtest.git/plain/doc/README.package-tests.rst>',
+        'description' => 'Enable a test suite to be used with this package. For more details see L<README.package-tests.rst|https://anonscm.debian.org/cgit/autopkgtest/autopkgtest.git/plain/doc/README.package-tests.rst>',
         'status' => 'deprecated',
         'summary' => 'name of the non regression test suite',
         'type' => 'leaf',
@@ -92,7 +92,17 @@ area/section if the package is in the contrib or non-free archive areas.
       },
       'Testsuite',
       {
-        'description' => 'Enable a testsuite to be used with this package. For more details see L<README.package-tests.rst|https://anonscm.debian.org/cgit/autopkgtest/autopkgtest.git/plain/doc/README.package-tests.rst>',
+        'compute' => {
+          'allow_override' => '1',
+          'formula' => 'my ($str) = ($team =~ /pkg-(perl|ruby)/);
+$str ? \'autopkgtest-pkg-\'.$str : undef;
+',
+          'use_eval' => '1',
+          'variables' => {
+            'team' => '- Maintainer'
+          }
+        },
+        'description' => 'Enable a test suite to be used with this package. For more details see L<README.package-tests.rst|https://anonscm.debian.org/cgit/autopkgtest/autopkgtest.git/plain/doc/README.package-tests.rst>',
         'migrate_from' => {
           'formula' => '$xs_testsuite',
           'variables' => {
@@ -102,6 +112,15 @@ area/section if the package is in the contrib or non-free archive areas.
         'summary' => 'name of the non regression test suite',
         'type' => 'leaf',
         'value_type' => 'uniline',
+        'warn_unless' => {
+          'team-test' => {
+            'code' => 'my $m = $self->grab_value(\'- Maintainer\');
+my ($team) = ( $m =~ /(pkg-(?:perl|ruby))/ );
+not defined $team or $_ eq \'autopkgtest-\'.$team ;',
+            'fix' => '$_ = undef; # restore default value',
+            'msg' => 'Wrong team value'
+          }
+        },
         'warn_unless_match' => {
           '^autopkgtest(-pkg-(perl|ruby))?$' => {
             'fix' => '$_ = undef; # restore default value',
@@ -167,7 +186,25 @@ in Debian developer reference.',
             }
           }
         },
+        'description' => 'List of packages that must be installed:
+
+'.'=over
+
+'.'=item *
+
+during clean
+
+'.'=item *
+
+to build architecture-dependent binaries ("Architecture: any" or specific architectures).
+
+'.'=back
+
+Technically, these packages must be installed for the following build targets: clean, build-arch, and binary-arch. See L<build target|https://www.debian.org/doc/debian-policy/ch-source.html#s-debianrules> doc and L<details on this field|https://www.debian.org/doc/debian-policy/ch-relationships.html#s-sourcebinarydeps>.
+
+On the other hand, the list of packages that must be installed to build architecture-independent binaries ("Architecture: all") should be listed in "Build-Depends-Indep" field.',
         'duplicates' => 'warn',
+        'summary' => 'List of package required during clean or build of architecture-dependent packages',
         'type' => 'list'
       },
       'Build-Depends-Indep',
@@ -177,7 +214,13 @@ in Debian developer reference.',
           'type' => 'leaf',
           'value_type' => 'uniline'
         },
+        'description' => 'List of packages that must be installed to build architecture-independent binaries ("Architecture: all").
+
+Technically, these packages must be installed for the following build targets: build, build-indep, binary, and binary-indep. See L<build target|https://www.debian.org/doc/debian-policy/ch-source.html#s-debianrules> doc and L<details on this field|https://www.debian.org/doc/debian-policy/ch-relationships.html#s-sourcebinarydeps>.
+
+Note that packages required during "clean" phase must be declared in "Build-Depends" field.',
         'duplicates' => 'warn',
+        'summary' => 'List of package required during build of architecture-independent package',
         'type' => 'list'
       },
       'Build-Conflicts',
@@ -186,12 +229,46 @@ in Debian developer reference.',
           'type' => 'leaf',
           'value_type' => 'uniline'
         },
+        'description' => 'List of packages that must be missing (i.e. B<not> installed):
+
+'.'=over
+
+'.'=item *
+
+during clean
+
+'.'=item *
+
+to build architecture-dependent binaries ("Architecture: any" or specific architectures).
+
+'.'=back
+
+Technically, these packages must B<not> be installed for the following build targets: clean, build-arch, and binary-arch. See L<build target|https://www.debian.org/doc/debian-policy/ch-source.html#s-debianrules> doc and L<details on this field|https://www.debian.org/doc/debian-policy/ch-relationships.html#s-sourcebinarydeps>.
+
+On the other hand, the list of packages that must B<not> be installed to build architecture-independent binaries ("Architecture: all") should be listed in "Build-Conflicts-Indep" field.',
+        'duplicates' => 'warn',
+        'summary' => 'List of package not wanted during clean or build of architecture-dependent packages',
+        'type' => 'list'
+      },
+      'Build-Conflicts-Indep',
+      {
+        'cargo' => {
+          'type' => 'leaf',
+          'value_type' => 'uniline'
+        },
+        'description' => 'List of packages that must be missing (i.e. B<not> installed) to build binaries with arch set to "all.
+
+Technically, these packages must B<not> be installed for the following build targets: build, build-indep, binary, and binary-indep. See L<build target|https://www.debian.org/doc/debian-policy/ch-source.html#s-debianrules> doc and L<details on this field|https://www.debian.org/doc/debian-policy/ch-relationships.html#s-sourcebinarydeps>.
+
+Note that packages not wanted during "clean" phase must be declared in "Build-Conflicts" field.',
+        'duplicates' => 'warn',
+        'summary' => 'List of package not wanted during build of architecture-independent packages',
         'type' => 'list'
       },
       'Standards-Version',
       {
         'default' => '3.9.6',
-        'description' => 'This field indicates the debian policy version number this package complies to',
+        'description' => 'This field indicates the Debian policy version number this package complies to',
         'match' => '\\d+\\.\\d+\\.\\d+(\\.\\d+)?',
         'summary' => 'Debian policy version number this package complies to',
         'type' => 'leaf',
