@@ -7,6 +7,18 @@
       '2010,2011 Dominique Dumont'
     ],
     'element' => [
+      'full_license',
+      {
+        'description' => 'if left blank here, the file must include a stand-alone License section matching each license short name listed on the first line (see the Standalone License Section section). Otherwise, this field should either include the full text of the license(s) or include a pointer to the license file under /usr/share/common-licenses. This field should include all text needed in order to fulfill both Debian Policy requirement for including a copy of the software distribution license, and any license requirements to include warranty disclaimers or other notices with the binary package.
+',
+        'type' => 'leaf',
+        'value_type' => 'string',
+        'warn_if_match' => {
+          'fill license' => {
+            'msg' => 'License contains copyright scanner boilerplate. Please update this field with the actual license text'
+          }
+        }
+      },
       'short_name',
       {
         'description' => 'abbreviated name for the license. If empty, it is given the default 
@@ -89,13 +101,6 @@ license: abbrev exception(?)
             }
           ]
         }
-      },
-      'full_license',
-      {
-        'description' => 'if left blank here, the file must include a stand-alone License section matching each license short name listed on the first line (see the Standalone License Section section). Otherwise, this field should either include the full text of the license(s) or include a pointer to the license file under /usr/share/common-licenses. This field should include all text needed in order to fulfill both Debian Policy requirement for including a copy of the software distribution license, and any license requirements to include warranty disclaimers or other notices with the binary package.
-',
-        'type' => 'leaf',
-        'value_type' => 'string'
       }
     ],
     'license' => 'LGPL2',
