@@ -1,6 +1,7 @@
 use Data::Dumper;
 use IO::File;
-use utf8; 
+use utf8;
+use Path::Tiny;
 
 $conf_file_name = "copyright";
 $conf_dir       = 'debian';
@@ -139,7 +140,7 @@ $skip = ( $@ or not -r '/etc/debian_version') ? 1 : 0 ;
         load_warnings => [ qr/Format does not match/ ],
         apply_fix => 1,
         dump_errors =>  [ 
-            qr/mandatory/ => 'Files:"*" Copyright:0="(c) foobar"',
+            qr/mandatory/ => 'Files:"*" Copyright="(c) foobar"',
             qr/mandatory/ => ' License:FOO text="foo bar" ! Files:"*" License short_name="FOO" '
         ],
     },
@@ -238,6 +239,9 @@ in ‘/usr/share/common-licenses/GPL-1’.",
         # Debian bug #721672
         name => 'file-instead-of-files' ,
 
+        # warning brought by fix of Debian #789568
+        load_warnings => [ qr/confusingly close/ ],
+
         check => {
             "Files:debian/patches/half_code_pod_errors.patch Copyright" =>
             '2010, Frank Wiegand <fwie@cpan.org>',
@@ -252,6 +256,36 @@ in ‘/usr/share/common-licenses/GPL-1’.",
 #            '2010, Frank Wiegand <fwie@cpan.org>',
         },
     },
+
+    {
+        name => 'update-from-scratch',
+        update => { in => path('t/scanner/examples/pan.in') },
+        check => {
+            "License:GPL-2 text" => {value => undef, mode => 'custom'},
+            "License:GPL-2 text" => qr/GNU/,
+        },
+        wr_check => {
+            "License:GPL-2 text" => {value => undef, mode => 'custom'},
+            "License:GPL-2 text" => qr/GNU/,
+        }
+    },
+
+    {
+        name => 'moarvm-from-scratch',
+        update => { in => path('t/scanner/examples/moarvm.in') },
+        check => {
+            'License:ISC text' => qr/Please fill/,
+            'License:BSD-2-clause text' => qr/Please fill/,
+        },
+        wr_check => {
+            qq!Files:"3rdparty/dyncall/*" License full_license! => undef,
+        },
+        file_contents_unlike => {
+            'debian/copyright' => qr/\n\nLicense: ISC\n\n/,
+            'debian/copyright' => qr/\n\nLicense: BSD-2-clause\n\n/,
+        },
+    },
+
 );
 
 1;
